@@ -1,8 +1,9 @@
 'use strict';
 
-var jade = require('../');
 var assert = require('assert');
 var fs = require('fs');
+var path = require('path');
+var jade = require('../');
 
 var perfTest = fs.readFileSync(__dirname + '/fixtures/perf.jade', 'utf8')
 
@@ -36,11 +37,11 @@ describe('jade', function(){
     });
 
     it('should support line endings', function(){
-      var str = [
+      var src = [
           'p',
           'div',
           'img'
-      ].join('\r\n');
+      ];
 
       var html = [
           '<p></p>',
@@ -48,35 +49,19 @@ describe('jade', function(){
           '<img/>'
       ].join('');
 
-      assert.equal(html, jade.render(str));
+      assert.equal(html, jade.render(src.join('\n')));
+      assert.equal(html, jade.render(src.join('\r')));
+      assert.equal(html, jade.render(src.join('\r\n')));
 
-      var str = [
-          'p',
-          'div',
-          'img'
-      ].join('\r');
-
-      var html = [
-          '<p></p>',
-          '<div></div>',
-          '<img/>'
-      ].join('');
-
-      assert.equal(html, jade.render(str));
-
-      var str = [
-          'p',
-          'div',
-          'img'
-      ].join('\r\n');
-
-      var html = [
+      html = [
           '<p></p>',
           '<div></div>',
           '<img>'
       ].join('');
 
-      assert.equal(html, jade.render(str, { doctype:'html' }));
+      assert.equal(html, jade.render(src.join('\n'), { doctype:'html' }));
+      assert.equal(html, jade.render(src.join('\r'), { doctype:'html' }));
+      assert.equal(html, jade.render(src.join('\r\n'), { doctype:'html' }));
     });
 
     it('should support single quotes', function(){
@@ -933,7 +918,11 @@ describe('jade', function(){
 
     it('should be reasonably fast', function(){
       jade.compile(perfTest, {})
-    })
+    });
+    it('allows trailing space (see #1586)', function () {
+      var res = jade.render('ul \n  li An Item');
+      assert.equal('<ul> <li>An Item</li></ul>', res);
+    });
   });
 
   describe('.renderFile()', function () {
@@ -974,6 +963,13 @@ describe('jade', function(){
       var actual = fn({name: 'foo'}).replace(/\s/g, '');
       assert(actual === expected);
     });
+    it('accepts the `name` option to rename the resulting function', function () {
+      var src = jade.compileFileClient(__dirname + '/cases/basic.jade', {name: 'myTemplateName'});
+      var expected = fs.readFileSync(__dirname + '/cases/basic.html', 'utf8').replace(/\s/g, '');
+      var fn = Function('jade', src + '\nreturn myTemplateName;')(jade.runtime);
+      var actual = fn({name: 'foo'}).replace(/\s/g, '');
+      assert(actual === expected);
+    });
   });
 
   describe('.runtime', function () {
@@ -993,4 +989,70 @@ describe('jade', function(){
       });
     });
   });
+
+  describe('filter indentation', function () {
+    it('is maintained', function () {
+      jade.filters.indents = function(str){
+        return str.split(/\n/).map(function (line) { return line.match(/^ */)[0].length; }).join(",");
+      };
+
+      var indents = [
+        ':indents',
+        '  x',
+        '   x',
+        '    x',
+        '     x',
+        '  x',
+        '      x',
+        '      x',
+        '     x',
+        '     x',
+        '      x',
+        '    x',
+        '  x',
+        '    x',
+        '  x',
+        '   x'
+      ].join('\n');
+
+      assert.equal(jade.render(indents), '0,1,2,3,0,4,4,3,3,4,2,0,2,0,1');
+    });
+  });
+
+  describe('.compile().dependencies', function() {
+    it('should list the filename of the template referenced by extends', function(){
+      var filename = __dirname + '/dependencies/extends1.jade';
+      var str = fs.readFileSync(filename, 'utf8');
+      var info = jade.compile(str, {filename: filename});
+      assert.deepEqual([
+        path.resolve(__dirname + '/dependencies/dependency1.jade')
+      ], info.dependencies);
+    });
+    it('should list the filename of the template referenced by an include', function() {
+      var filename = __dirname + '/dependencies/include1.jade';
+      var str = fs.readFileSync(filename, 'utf8');
+      var info = jade.compile(str, {filename: filename});
+      assert.deepEqual([
+        path.resolve(__dirname + '/dependencies/dependency1.jade')
+      ], info.dependencies);
+    });
+    it('should list the dependencies of extends dependencies', function() {
+      var filename = __dirname + '/dependencies/extends2.jade';
+      var str = fs.readFileSync(filename, 'utf8');
+      var info = jade.compile(str, {filename: filename});
+      assert.deepEqual([
+        path.resolve(__dirname + '/dependencies/dependency2.jade'),
+        path.resolve(__dirname + '/dependencies/dependency3.jade')
+      ], info.dependencies);
+    });
+    it('should list the dependencies of include dependencies', function() {
+      var filename = __dirname + '/dependencies/include2.jade';
+      var str = fs.readFileSync(filename, 'utf8');
+      var info = jade.compile(str, {filename: filename});
+      assert.deepEqual([
+        path.resolve(__dirname + '/dependencies/dependency2.jade'),
+        path.resolve(__dirname + '/dependencies/dependency3.jade')
+      ],info.dependencies);
+    });
+  });
 });
